(function() {
  var async, autoSpec, autospec, checkPaths, coffee, combineProd, commonJsMock, config, displayNotification, engine, exec, fs, gatherFiles, growl, handleError, intialTestComplete, libDir, main, path, prodCoffeeFiles, prodCoffeeOpts, prodDeployDir, prodJsFiles, prodSrcDir, prodTargetCoffeeFile, prodTargetCreate, prodTargetFileName, prodTargetJsFile, prodTargetJsMinFile, rootDir, runTests, showCoverageReport, spec, specCoffeeFiles, specCoffeeOpts, specFileDir, specJsFiles, stalker, test, uglify, unWatchCoffeeFile, util, walk, watchCoffeeFile, writeSpecFile, _ref, _ref2, _ref3, _ref4, _ref5;
  fs = require('fs');
  exec = require('child_process').exec;
  util = require('util');
  uglify = require('uglify-js');
  stalker = require('stalker');
  growl = require('growl');
  path = require('path');
  async = require('async');
  libDir = path.join(path.dirname(fs.realpathSync(__filename)), '../lib');
  rootDir = process.cwd();
  config = (function() {
    try {
      return require("" + rootDir + "/dependency/kitkat_config.js");
    } catch (_e) {}
  })();
  config = config != null ? config : config = {};
  prodSrcDir = (_ref = config.prodSrcDir) != null ? _ref : config.prodSrcDir = "" + rootDir + "/src";
  specFileDir = (_ref2 = config.specFileDir) != null ? _ref2 : config.specFileDir = "" + rootDir + "/spec";
  prodDeployDir = (_ref3 = config.prodDeployDir) != null ? _ref3 : config.prodDeployDir = "" + rootDir + "/lib";
  commonJsMock = (_ref4 = config.commonJsMock) != null ? _ref4 : config.commonJsMock = "/mocks/commonJs.js";
  prodTargetCreate = (_ref5 = config.prodTargetCreate) != null ? _ref5 : config.prodTargetCreate = false;
  prodTargetFileName = path.basename(rootDir);
  prodTargetCoffeeFile = "" + prodDeployDir + "/" + prodTargetFileName + ".app.coffee";
  prodTargetJsFile = "" + prodDeployDir + "/" + prodTargetFileName + ".app.js";
  prodTargetJsMinFile = "" + prodDeployDir + "/" + prodTargetFileName + ".min.js";
  engine = {};
  engine.jasmine_node = path.join(path.dirname(fs.realpathSync(__filename)), '../node_modules/jasmine-node/bin/jasmine-node spec/');
  engine.rhino = "" + libDir + "/env_jasmine/bin/run_test.sh --projectPath=" + rootDir + " ";
  engine.expresso = path.join(path.dirname(fs.realpathSync(__filename)), '../node_modules/expresso/bin/expresso spec/*.coffee -I lib');
  engine.type = "expresso";
  engine.execute = engine.expresso;
  prodCoffeeFiles = [];
  specCoffeeFiles = [];
  prodJsFiles = [];
  specJsFiles = [];
  intialTestComplete = false;
  autoSpec = false;
  showCoverageReport = false;
  prodCoffeeOpts = "--bare --output " + prodDeployDir + " --compile " + prodTargetCoffeeFile;
  specCoffeeOpts = "--output " + specFileDir;
  checkPaths = function(callback) {
    var mockDir;
    path.exists(prodDeployDir, function(exists) {
      if (!exists) {
        return exec("mkdir " + prodDeployDir, function(err, stdout, stderr) {
          if (err) {
            return callback(err);
          }
        });
      }
    });
    path.exists(specFileDir, function(exists) {
      if (!exists) {
        return exec("mkdir " + specFileDir, function(err, stdout, stderr) {
          if (err) {
            return callback(err);
          }
        });
      }
    });
    path.exists(prodSrcDir, function(exists) {
      if (!exists) {
        return exec("mkdir " + prodSrcDir, function(err, stdout, stderr) {
          if (err) {
            return callback(err);
          }
        });
      }
    });
    mockDir = rootDir + path.dirname(commonJsMock);
    path.exists(rootDir + commonJsMock, function(exists) {
      if (!exists) {
        return exec("mkdir " + mockDir, function(err, stdout, stderr) {
          if (err) {
            return callback(err);
          } else {
            return exec("cp " + libDir + "/commonjs.js " + mockDir + "/commonjs.js", function(err, stdout, stderr) {
              if (err) {
                return callback(err);
              }
            });
          }
        });
      }
    });
    return callback;
  };
  exports.run = function() {
    var args, command, commands, key, optimist, value, _ref6, _ref7;
    checkPaths(function(err) {
      if (err) {
        handleError(err);
      }
    });
    optimist = require('optimist');
    args = optimist.argv;
    optimist.alias('test', 't');
    optimist.alias('spec', 's');
    optimist.alias('autospec', 'a');
    optimist.alias('engine', 'e');
    optimist.alias('uglify', 'u');
    optimist.alias('coverage', 'c');
    optimist.usage("Kitkat - Kontinuous Integrated Testing Koffee Application Template\n\n default usage with no arguments monitors current project for changes and runs all spec files found against all coffee files found.", {
      'test': {
        description: 'runs all tests against your application js files in the spec\n\t\tpath unless overridden in the dependency config file\n',
        required: false
      },
      'spec': {
        description: 'generates spec files based on the supplied path/file, if none\n\t\tprovided then processes all coffee files in project\n',
        required: false
      },
      'autospec': {
        description: 'generates new spec files and monitors all files for changes,\n\t\tautomatically runs tests on startup and as files change\n',
        required: false
      },
      'engine': {
        description: 'specifies the testing engine to use, by default we will use expresso\n\t\tavailable options include "expresso", "rhino", and "jasmine-node"\n',
        required: false
      },
      'ugilfy': {
        description: 'creates a combined source file then minifies it using uglify-js library',
        required: false
      },
      'coverage': {
        description: 'generates the code coverage report at the end of testing',
        required: false
      }
    });
    commands = [];
    for (key in args) {
      value = args[key];
      if (key !== "$0" && key !== "_") {
        try {
          switch (key.toLowerCase()) {
            case "t":
            case "test":
            case "s":
            case "spec":
            case "a":
            case "autotest":
              commands.push(key);
              break;
            case "e":
            case "engine":
              engine.type = value != null ? typeof value.toLowerCase === "function" ? value.toLowerCase() : void 0 : void 0;
              switch (engine.type) {
                case "expresso":
                  engine.execute = engine.expresso;
                  break;
                case "rhino":
                  engine.execute = engine.rhino;
                  break;
                case "jasmine-node":
                  engine.execute = engine.jasmine_node;
                  break;
                default:
                  throw "Invalid engine";
              }
              break;
            case "u":
            case "uglify":
              prodTargetCreate = true;
              break;
            case "c":
            case "coverage":
              showCoverageReport = true;
              engine.expresso = engine.expresso + " --coverage";
              if (engine.type === "expresso") {
                engine.execute = engine.expresso;
              }
              break;
            default:
              throw "Invaid argument";
          }
        } catch (ex) {
          util.log("" + ex + "\n");
          optimist.showHelp();
          return;
        }
      }
    }
    command = (_ref6 = commands[0]) != null ? _ref6.toLowerCase() : void 0;
    switch (command) {
      case "t":
      case "test":
        util.log("Initiating tests.......");
        return test();
      case "s":
      case "spec":
        util.log("Generating spec files......");
        return spec((_ref7 = args.s) != null ? _ref7 : args.s = args.spec);
      case "a":
      case "autospec":
        util.log("Autospec and testing current project.....");
        return autospec();
      default:
        util.log("Monitoring and testing current project.....");
        return main();
    }
  };
  autospec = function() {
    autoSpec = true;
    return main();
  };
  coffee = function(options, file) {
    if (options == null) {
      options = "";
    }
    util.log("Compiling " + file);
    return exec("coffee " + options + " --compile " + file, function(err, stdout, stderr) {
      if (err) {
        return handleError(err);
      }
    });
  };
  combineProd = function(createCombined, callback) {
    var appContents, file, index, process, remaining, _len;
    if (createCombined == null) {
      createCombined = false;
    }
    appContents = new Array(remaining = prodCoffeeFiles.length);
    util.log("Appending " + prodCoffeeFiles.length + " files to " + prodTargetCoffeeFile);
    for (index = 0, _len = prodCoffeeFiles.length; index < _len; index++) {
      file = prodCoffeeFiles[index];
      coffee("--output " + prodDeployDir, "" + file);
      if (createCombined) {
        fs.readFile("" + file, 'utf8', function(err, fileContents) {
          if (err) {
            callback(err);
          }
          appContents[index] = fileContents;
          if (--remaining === 0) {
            return process();
          }
        });
      } else {
        if (--remaining === 0) {
          callback(null);
        }
      }
    }
    return process = function() {
      fs.writeFile(prodTargetCoffeeFile, appContents.join('\n\n'), 'utf8', function(err) {
        if (err) {
          return callback(err);
        }
      });
      return exec("coffee " + prodCoffeeOpts, function(err, stdout, stderr) {
        var message;
        if (err) {
          callback(err);
        }
        message = "Compiled " + prodTargetJsFile;
        util.log(message);
        return callback(null);
      });
    };
  };
  displayNotification = function(message) {
    var options;
    if (message == null) {
      message = '';
    }
    options = {
      title: 'CoffeeScript',
      image: 'lib/CoffeeScript.png'
    };
    return growl.notify(message, options);
  };
  gatherFiles = function(filepath, extRegex, callback) {
    var file, filesFound;
    filesFound = [];
    file = path.normalize(filepath);
    return fs.stat(file, function(err, stats) {
      if (err) {
        file += extRegex.toString().replace(/\//g, "").replace(/\\/g, "");
        return fs.stat(file, function(err, stats) {
          if (err) {
            callback(err, null);
          }
          if (!err) {
            filesFound.push(file);
            return callback(null, filesFound);
          }
        });
      } else {
        if (stats.isDirectory()) {
          return walk(path.normalize(file + '/'), extRegex, function(err, results) {
            if (err) {
              callback(err, null);
            }
            if (!err) {
              filesFound = results;
              return callback(null, filesFound);
            }
          });
        } else {
          filesFound.push(file);
          return callback(null, filesFound);
        }
      }
    });
  };
  handleError = function(error) {
    util.log(error);
    return displayNotification(error);
  };
  main = function() {
    stalker.watch("" + prodSrcDir, watchCoffeeFile, unWatchCoffeeFile);
    stalker.watch("" + specFileDir, watchCoffeeFile, unWatchCoffeeFile);
    return test("/");
  };
  runTests = function(callback) {
    var message;
    message = "";
    return exec(engine.execute, function(err, stdout, stderr) {
      var assertion, fail, failure, options, output, pass, test, total, _ref6, _ref7;
      util.log(engine.execute);
      switch (engine.type) {
        case "rhino":
          pass = stdout.match(/Passed: \d+/);
          fail = stdout.match(/Failed: \d+/);
          total = stdout.match(/Total : \d+/);
          message = "" + pass + " " + fail + " " + total;
          util.log(stdout);
          break;
        case "jasmine-node":
          message = "" + stdout.match(/.*failure/).toString();
          output = stdout.toString();
          output = output.replace(/at .*\n/g, "");
          test = message.match(/\d+ test/);
          assertion = message.match(/\d+ assertion/);
          failure = message.match(/\d+ failure/);
          message = "" + test + "s, " + assertion + "s, " + failure;
          util.log(output);
          break;
        default:
          if (err) {
            message = "" + ((_ref6 = stderr.match(/Failures.*/)) != null ? _ref6.toString() : void 0);
            message = message.replace(/\[\d*m/g, "");
            output = stderr.toString();
            output = output.replace(/at .*\n/g, "");
            util.log(output);
          } else {
            message = "100% - " + ((_ref7 = stderr.match(/\d* tests/)) != null ? _ref7.toString() : void 0) + " passed";
          }
      }
      if (err) {
        options = {
          title: 'Automated Testing',
          image: "" + libDir + "/error.png"
        };
      } else {
        options = {
          title: 'Automated Testing',
          image: "" + libDir + "/success.png"
        };
      }
      growl.notify(message, options);
      return callback(null);
    });
  };
  spec = function(filepath) {
    return async.series({
      gather_files: function(callback) {
        if (filepath === true) {
          filepath = "/";
        }
        filepath = path.join(prodSrcDir, path.normalize(filepath));
        return gatherFiles(filepath, /\.coffee/, function(err, results) {
          prodCoffeeFiles = results;
          return callback(err, results);
        });
      },
      create_specs: function(callback) {
        var file, index, pending, _len, _results;
        pending = prodCoffeeFiles.length;
        _results = [];
        for (index = 0, _len = prodCoffeeFiles.length; index < _len; index++) {
          file = prodCoffeeFiles[index];
          _results.push((function(file, index) {
            var specFile;
            specFile = path.join(specFileDir, path.basename(file, '.coffee') + ".spec.coffee");
            fs.stat(specFile, function(err, stats) {
              var prodFile;
              if (err) {
                prodFile = path.join(prodDeployDir, path.basename(file, '.coffee') + ".js");
                return writeSpecFile(specFile, prodFile, function(err, results) {
                  if (err) {
                    return callback(err, null);
                  }
                });
              }
            });
            if (!--pending) {
              return callback(null, null);
            }
          })(file, index));
        }
        return _results;
      }
    }, function(err, results) {
      if (err) {
        return handleError(err);
      }
    });
  };
  test = function() {
    return async.series({
      gather_files: function(callback) {
        return async.parallel({
          gather_specs: function(callback) {
            return gatherFiles(specFileDir, /\.spec\.coffee/, function(err, results) {
              specCoffeeFiles = results;
              return callback(err, results);
            });
          },
          gather_prod: function(callback) {
            return gatherFiles(prodSrcDir, /\.coffee/, function(err, results) {
              prodCoffeeFiles = results;
              return callback(err, results);
            });
          }
        }, function(err, results) {
          return callback(err, results);
        });
      },
      create_specs: function(callback) {
        var file, index, pending, _len, _results;
        if (autoSpec) {
          pending = prodCoffeeFiles.length;
          _results = [];
          for (index = 0, _len = prodCoffeeFiles.length; index < _len; index++) {
            file = prodCoffeeFiles[index];
            _results.push((function(file, index) {
              var specFile;
              specFile = path.join(specFileDir, path.basename(file, '.coffee') + ".spec.coffee");
              fs.stat(specFile, function(err, stats) {
                var prodFile;
                if (err) {
                  prodFile = path.join(prodDeployDir, path.basename(file, '.coffee') + ".js");
                  writeSpecFile(specFile, prodFile, function(err, results) {
                    if (err) {
                      return callback(err, null);
                    }
                  });
                  return specCoffeeFiles.push(specFile);
                }
              });
              if (!--pending) {
                return callback(null, null);
              }
            })(file, index));
          }
          return _results;
        } else {
          return callback(null, null);
        }
      },
      combine_prod: function(callback) {
        return combineProd(prodTargetCreate, function(err) {
          return callback(err, null);
        });
      },
      compile_specs: function(callback) {
        var file, index, pending, _len, _results;
        pending = specCoffeeFiles.length;
        _results = [];
        for (index = 0, _len = specCoffeeFiles.length; index < _len; index++) {
          file = specCoffeeFiles[index];
          _results.push((function(file, index) {
            coffee(specCoffeeOpts, "" + file);
            if (!--pending) {
              return callback(null, null);
            }
          })(file, index));
        }
        return _results;
      },
      run_specs: function(callback) {
        return runTests(function(err) {
          intialTestComplete = true;
          return callback(err, null);
        });
      },
      uglifyIt: function(callback) {
        var jsp, pro;
        if (!prodTargetCreate) {
          return callback(null, null);
        } else {
          jsp = uglify.parser;
          pro = uglify.uglify;
          fs.readFile(prodTargetJsFile, 'utf8', function(err, fileContents) {
            var ast, final_code, message;
            ast = jsp.parse(fileContents);
            ast = pro.ast_mangle(ast);
            ast = pro.ast_squeeze(ast);
            final_code = pro.gen_code(ast);
            fs.writeFile(prodTargetJsMinFile, final_code);
            message = "Uglified " + prodTargetJsMinFile;
            return util.log(message);
          });
          return callback(null, null);
        }
      }
    }, function(err, results) {
      if (err) {
        return handleError(err);
      }
    });
  };
  unWatchCoffeeFile = function(err, file) {
    if (("" + file).match(/\.coffee/) != null) {
      util.log("" + file + " removed.");
      fs.unwatchFile("" + file);
      return runTests(function(err) {});
    }
  };
  walk = function(dir, extRegex, callback) {
    var pending, results;
    results = [];
    pending = 0;
    return fs.readdir(dir, function(err, list) {
      var file, _i, _len, _results;
      if (err) {
        callback(err, null);
      }
      pending = list.length;
      _results = [];
      for (_i = 0, _len = list.length; _i < _len; _i++) {
        file = list[_i];
        _results.push((function(file) {
          file = dir + file;
          return fs.stat(file, function(err, stat) {
            if (stat && stat.isDirectory()) {
              file += "/";
              if (("" + file).match(/node_modules/) != null) {
                if (!--pending) {
                  return callback(null, results);
                }
              } else {
                return walk(file, extRegex, function(err, res) {
                  results = results.concat(res);
                  if (!--pending) {
                    return callback(null, results);
                  }
                });
              }
            } else {
              if (file.match(extRegex)) {
                results.push(file);
              }
              if (!--pending) {
                return callback(null, results);
              }
            }
          });
        })(file));
      }
      return _results;
    });
  };
  watchCoffeeFile = function(err, file) {
    if (!(("" + file).match(/\.spec\.coffee/) != null) && (("" + file).match(/\.coffee/) != null)) {
      if (autoSpec === true && intialTestComplete === true) {
        spec("/");
      }
    }
    if (("" + file).match(/\.coffee/) != null) {
      util.log("Watch added for " + file);
      fs.unwatchFile("" + file);
      return fs.watchFile("" + file, function(curr, prev) {
        if (+curr.mtime !== +prev.mtime) {
          util.log("Saw change in " + file);
          if (("" + file).match(/\.spec\.coffee/)) {
            coffee(specCoffeeOpts, "" + file);
          } else {
            coffee("--output " + prodDeployDir, "" + file);
          }
          return runTests(function(err) {});
        }
      });
    }
  };
  writeSpecFile = function(specFile, prodFile, callback) {
    var libFile, specContent;
    libFile = path.basename(prodFile, path.extname(prodFile));
    switch (engine.type) {
      case "rhino":
        specContent = ["EnvJasmine.load (EnvJasmine.projectPath + \"", "EnvJasmine.load (EnvJasmine.projectPath + \"", "describe 'default spec suite', ->", "\tit 'needs to be modified with real test cases and thus fails until this file is modified.', ->", "\t\t(expect false).toBeTruthy()"];
        specContent[0] = specContent[0] + commonJsMock + "\")";
        specContent[1] = specContent[1] + prodDeployDir.replace(rootDir, "") + "/" + path.basename(prodFile) + "\")";
        break;
      case "jasmine-node":
        specContent = ["replace with libfile", "describe 'default spec suite', ->", "\tit 'needs to be modified with real test cases and thus fails until this file is modified.', ->", "\t\t(expect false).toBeTruthy()"];
        specContent[0] = "" + libFile + " = require('../lib/" + libFile + "')";
        break;
      default:
        specContent = ["replace with libfile", "assert = require('assert')", "module.exports =", "\t\"You need to modify this with real test cases and thus it fails until this file is modified.\": ->", "\t\tassert.ok false, \"it will fail\""];
        specContent[0] = "" + libFile + " = require('" + libFile + "')";
    }
    fs.writeFile(specFile, specContent.join('\n'), 'utf8', function(err) {
      if (err) {
        callback(err, null);
      }
      return util.log("Created specfile '" + specFile + "'");
    });
    return callback(null, null);
  };
}).call(this);
